package com.example.docapi_test;

import androidx.annotation.RequiresApi;
import androidx.appcompat.app.AppCompatActivity;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.fri.fridecompose.CardAPI;
import com.fri.fridecompose.CardInfoStruct;

import java.nio.charset.StandardCharsets;

public class MainActivity extends AppCompatActivity {

    private TextView tv_idInfo;
    private Button bt_open, bt_close;
    private ImageView IMG_Photo;

    private Thrcv rcv = null; // 读卡线程
    private boolean _usbrun = true; // usb 是否打开

    byte [] pucCHMsg = new byte[256];
    byte [] pucPHMsg = new byte[38892];
    byte [] pucFPMsg = new byte[1024];

    int [] puiFPMsgLen = new int[1];
    int []puiCHMsgLen = new int[1];
    int []puiPHMsgLen = new int[1];

    private CardAPI doc;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        tv_idInfo = findViewById(R.id.tv_idInfo);
        bt_open = findViewById(R.id.bt_open);
        bt_close = findViewById(R.id.bt_close);
        IMG_Photo = (ImageView) findViewById(R.id.imageView1);

        ButtonClickHandler buttonClickHandler = new ButtonClickHandler();
        bt_open.setOnClickListener(buttonClickHandler);
        bt_close.setOnClickListener(buttonClickHandler);

        try {
            doc = new CardAPI(this);
        } catch (Exception e) {
            if (e.getCause() == null) {
                Toast.makeText(MainActivity.this, "usb 设备异常或无连接", Toast.LENGTH_LONG).show();
                this.finish();
            } else {
                Toast.makeText(MainActivity.this, "usb未授权，请授权后重新打开", Toast.LENGTH_LONG).show();
                this.finish();
            }
        }
    }

    public class ButtonClickHandler implements View.OnClickListener {
        @Override
        public void onClick(View v) {
            if (v.getId() == R.id.bt_open) {

                _usbrun = true;
                rcv = new Thrcv();
                new Thread(rcv).start();
                Toast.makeText(MainActivity.this, "开始读卡", Toast.LENGTH_LONG).show();

            }else if (v.getId() == R.id.bt_close) {
                if (_usbrun) {
                    _usbrun = false;
                    Toast.makeText(MainActivity.this, "停止读卡", Toast.LENGTH_LONG).show();
                }
            }
        }
    }

    // 读卡线程
    public class Thrcv implements Runnable {
        public void run() {

            int ret = 0;
            while (_usbrun) {
                ret = doc.findIDCard();
                if(ret == 0x90) // 找卡成功 9F
                {
                    ret = ReadBaseMsgToStr();
                }
            }
        }
    }

    private Handler mHandler = new Handler() {
        // 在收到消息时触发
        public void handleMessage(Message msgs) {
            switch (msgs.what) {
                case 1:
                    tv_idInfo.setText(msgs.obj.toString());
                    break;
                case 2:
                    if (msgs.obj == null)
                        IMG_Photo.setImageBitmap(null);
                    else {
                        IMG_Photo.setImageBitmap((Bitmap) msgs.obj);
                        }
                    break;
                default:
                    break;
            }
        }
    };

    public void handle_message(Object obj, int param)
    {
        Message message = Message.obtain();
        message.obj = obj;
        message.what = param;
        mHandler.sendMessage(message);
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public int ReadBaseMsgToStr() {
        int ret;

        CardInfoStruct cardInfoStruct = new CardInfoStruct();

        ret = doc.readIDCard(cardInfoStruct, pucPHMsg, puiPHMsgLen, pucFPMsg, puiFPMsgLen, 1);
        String message = "";
        if (ret == 0x90) {
            handle_message("读卡成功", 1);
            if (cardInfoStruct.cardType == 1) {
                // 身份证
                message = "证件类型：身份证" + "\n"
                        + "姓名：" + new String(cardInfoStruct.sfz_name, StandardCharsets.UTF_16LE) + "\n"
                        + "证件号：" + new String(cardInfoStruct.sfz_idNumber, StandardCharsets.UTF_16LE) + "\n"
                        + "性别：" + new String(cardInfoStruct.sfz_gender, StandardCharsets.UTF_16LE) + "\n"
                        + "民族：" + new String(cardInfoStruct.sfz_nation, StandardCharsets.UTF_16LE) + "\n"
                        + "生日：" + new String(cardInfoStruct.sfz_birthday, StandardCharsets.UTF_16LE) + "\n"
                        + "地址：" + new String(cardInfoStruct.sfz_address, StandardCharsets.UTF_16LE) + "\n"
                        + "签发机关：" + new String(cardInfoStruct.sfz_issueOrganization, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期起始：" + new String(cardInfoStruct.sfz_validDateStart, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期截至：" + new String(cardInfoStruct.sfz_validDateEnd, StandardCharsets.UTF_16LE) + "\n";
                Bitmap bitmapPhoto = BitmapFactory.decodeByteArray(pucPHMsg, 0, 38862);
                handle_message(bitmapPhoto, 2);
            } else if (cardInfoStruct.cardType == 2) {
                // 港澳台居民居住证
                message = "证件类型：港澳证" + "\n"
                        + "姓名：" + new String(cardInfoStruct.gat_name, StandardCharsets.UTF_16LE) + "\n"
                        + "证件号：" + new String(cardInfoStruct.gat_idNumber, StandardCharsets.UTF_16LE) + "\n"
                        + "性别：" + new String(cardInfoStruct.gat_gender, StandardCharsets.UTF_16LE) + "\n"
                        + "通行证号：" + new String(cardInfoStruct.gat_passCheckId, StandardCharsets.UTF_16LE) + "\n"
                        + "生日：" + new String(cardInfoStruct.gat_birthday, StandardCharsets.UTF_16LE) + "\n"
                        + "地址：" + new String(cardInfoStruct.gat_address, StandardCharsets.UTF_16LE) + "\n"
                        + "签发机关：" + new String(cardInfoStruct.gat_issueOrganization, StandardCharsets.UTF_16LE) + "\n"
                        + "签发次数：" + new String(cardInfoStruct.gat_issuesNumber, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期起始：" + new String(cardInfoStruct.gat_validDateStart, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期截至：" + new String(cardInfoStruct.gat_validDateEnd, StandardCharsets.UTF_16LE) + "\n";
                Bitmap bitmapPhoto = BitmapFactory.decodeByteArray(pucPHMsg, 0, 38862);
                handle_message(bitmapPhoto, 2);
            } else if (cardInfoStruct.cardType == 3) {
                // 旧版永居证
                message = "证件类型：旧版永居证" + "\n"
                        + "英文姓名：" + new String(cardInfoStruct.yjzOld_nameEnglish, StandardCharsets.UTF_16LE) + "\n"
                        + "中文姓名：" + new String(cardInfoStruct.yjzOld_nameChinese, StandardCharsets.UTF_16LE) + "\n"
                        + "证件号：" + new String(cardInfoStruct.yjzOld_idNumber, StandardCharsets.UTF_16LE) + "\n"
                        + "性别：" + new String(cardInfoStruct.yjzOld_gender, StandardCharsets.UTF_16LE) + "\n"
                        + "国籍：" + new String(cardInfoStruct.yjzOld_nationality, StandardCharsets.UTF_16LE) + "\n"
                        + "生日：" + new String(cardInfoStruct.yjzOld_birthday, StandardCharsets.UTF_16LE) + "\n"
                        + "签发机关：" + new String(cardInfoStruct.yjzOld_issueOrganization, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期起始：" + new String(cardInfoStruct.yjzOld_validDateStart, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期截至：" + new String(cardInfoStruct.yjzOld_validDateEnd, StandardCharsets.UTF_16LE) + "\n";
                Bitmap bitmapPhoto = BitmapFactory.decodeByteArray(pucPHMsg, 0, 38862);
                handle_message(bitmapPhoto, 2);
            } else if (cardInfoStruct.cardType == 4) {
                // 新版永居证
                message = "证件类型：新版永居证" + "\n"
                        + "姓名：" + new String(cardInfoStruct.yjz_nameEnglish, StandardCharsets.UTF_16LE) + "\n"
                        + "英文姓名备用：" + new String(cardInfoStruct.yjz_nameEnglishAdd, StandardCharsets.UTF_16LE) + "\n"
                        + "中文姓名：" + new String(cardInfoStruct.yjz_nameChinese, StandardCharsets.UTF_16LE) + "\n"
                        + "证件号：" + new String(cardInfoStruct.yjz_idNumber, StandardCharsets.UTF_16LE) + "\n"
                        + "性别：" + new String(cardInfoStruct.yjz_gender, StandardCharsets.UTF_16LE) + "\n"
                        + "民族：" + new String(cardInfoStruct.yjz_nationality, StandardCharsets.UTF_16LE) + "\n"
                        + "生日：" + new String(cardInfoStruct.yjz_birthday, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期起始：" + new String(cardInfoStruct.yjz_validDateStart, StandardCharsets.UTF_16LE) + "\n"
                        + "有效期截至：" + new String(cardInfoStruct.yjz_validDateEnd, StandardCharsets.UTF_16LE) + "\n";
                Bitmap bitmapPhoto = BitmapFactory.decodeByteArray(pucPHMsg, 0, 38862);
                handle_message(bitmapPhoto, 2);
            } else {
                handle_message("无法识别的证件类型", 1);
            }

            handle_message(message, 1);
        } else {
            handle_message("读卡失败", 1);
        }
        return ret;
    }
}